@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    config.c for the Openbox window manager
-   Copyright (c) 2004        Mikael Magnusson
+   Copyright (c) 2006        Mikael Magnusson
    Copyright (c) 2003        Ben Jansens
 
    This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,8 @@
 #include "mouse.h"
 #include "prop.h"
 #include "translate.h"
+#include "client.h"
+#include "screen.h"
 #include "parser/parse.h"
 #include "openbox.h"
 
@@ -35,25 +37,30 @@ ObPlacePolicy config_place_policy;
 
 gchar   *config_theme;
 gboolean config_theme_keepborder;
+gboolean config_theme_hidedisabled;
 
-gchar *config_title_layout;
+gchar   *config_title_layout;
+gboolean config_title_number;
 
 gint    config_desktops_num;
 GSList *config_desktops_names;
 gint    config_screen_firstdesk;
 
 gboolean config_resize_redraw;
+gboolean config_resize_four_corners;
 gint     config_resize_popup_show;
 gint     config_resize_popup_pos;
 
 ObStackingLayer config_dock_layer;
 gboolean        config_dock_floating;
+gboolean        config_dock_nostrut;
 ObDirection     config_dock_pos;
 gint            config_dock_x;
 gint            config_dock_y;
 ObOrientation   config_dock_orient;
 gboolean        config_dock_hide;
 guint           config_dock_hide_delay;
+guint           config_dock_show_delay;
 guint           config_dock_app_move_button;
 guint           config_dock_app_move_modifiers;
 
@@ -65,13 +72,188 @@ gint config_mouse_dclicktime;
 
 gboolean config_menu_warppointer;
 gboolean config_menu_xorstyle;
-gboolean config_menu_hilightfirst;
 guint    config_menu_hide_delay;
+gboolean config_menu_middle;
+guint    config_submenu_show_delay;
+gboolean config_menu_client_list_icons;
 
 GSList *config_menu_files;
 
-gint config_resist_win;
-gint config_resist_edge;
+gint     config_resist_win;
+gint     config_resist_edge;
+gboolean config_resist_layers_below;
+
+GSList *config_per_app_settings;
+
+/*
+  <applications>
+    <application name="aterm">
+      <decor>false</decor>
+    </application>
+    <application name="Rhythmbox">
+      <layer>above</layer>
+      <position>
+        <x>700</x>
+        <y>0</y>
+      </position>
+      <head>1</head>
+    </application>
+  </applications>
+*/
+
+/* Manages settings for individual applications.
+   Some notes: head is the screen number in a multi monitor
+   (Xinerama) setup (starting from 0) or mouse, meaning the
+   head the pointer is on. Default: mouse.
+   Layer can be three values, above (Always on top), below
+   (Always on bottom) and everything else (normal behaviour).
+   Positions can be an integer value or center, which will
+   center the window in the specified axis. Position is relative
+   from head, so <position><x>center</x></position><head>1</head>
+   will center the window on the second head.
+*/
+static void parse_per_app_settings(ObParseInst *i, xmlDocPtr doc,
+                                   xmlNodePtr node, gpointer d)
+{
+    xmlNodePtr app = parse_find_node("application", node->children);
+    gchar *name, *class;
+    gboolean name_set, class_set;
+    gboolean x_pos_given;
+
+    while (app) {
+        name_set = class_set = x_pos_given = FALSE;
+
+        class_set = parse_attr_string("class", app, &class);
+        name_set = parse_attr_string("name", app, &name);
+        if (class_set || name_set) {
+            xmlNodePtr n, c;
+            ObAppSettings *settings = g_new0(ObAppSettings, 1);
+            
+            if (name_set)
+                settings->name = name;
+            else
+                settings->name = NULL;
+
+            if (class_set)
+                settings->class = class;
+            else
+                settings->class = NULL;
+
+            if (!parse_attr_string("role", app, &settings->role))
+                settings->role = NULL;
+
+            settings->decor = -1;
+            if ((n = parse_find_node("decor", app->children)))
+                settings->decor = parse_bool(doc, n);
+
+            settings->shade = -1;
+            if ((n = parse_find_node("shade", app->children)))
+                settings->shade = parse_bool(doc, n);
+
+            settings->position.x = settings->position.y = 0;
+            settings->pos_given = FALSE;
+            if ((n = parse_find_node("position", app->children))) {
+                if ((c = parse_find_node("x", n->children))) {
+                    gchar *s = parse_string(doc, c);
+                    if (!strcmp(s, "center")) {
+                        settings->center_x = TRUE;
+                        x_pos_given = TRUE;
+                    } else {
+                        settings->position.x = parse_int(doc, c);
+                        x_pos_given = TRUE;
+                    }
+                    g_free(s);
+                }
+
+                if (x_pos_given && (c = parse_find_node("y", n->children))) {
+                    gchar *s = parse_string(doc, c);
+                    if (!strcmp(s, "center")) {
+                        settings->center_y = TRUE;
+                        settings->pos_given = TRUE;
+                    } else {
+                        settings->position.y = parse_int(doc, c);
+                        settings->pos_given = TRUE;
+                    }
+                    g_free(s);
+                }
+            }
+
+            settings->focus = -1;
+            if ((n = parse_find_node("focus", app->children)))
+                settings->focus = parse_bool(doc, n);
+
+            if ((n = parse_find_node("desktop", app->children))) {
+                gchar *s = parse_string(doc, n);
+                if (!strcmp(s, "all"))
+                    settings->desktop = DESKTOP_ALL;
+                else
+                    settings->desktop = parse_int(doc, n);
+                g_free(s);
+            } else
+                settings->desktop = DESKTOP_ALL - 1; /* lets hope the user
+                                                      * doesn't have 2^32
+                                                      * desktops */
+
+            if ((n = parse_find_node("head", app->children))) {
+                gchar *s = parse_string(doc, n);
+                if (!strcmp(s, "mouse"))
+                    settings->head = -1;
+                else
+                    settings->head = parse_int(doc, n);
+                g_free(s);
+            }
+
+            settings->layer = -2;
+            if ((n = parse_find_node("layer", app->children))) {
+                gchar *s = parse_string(doc, n);
+                if (!strcmp(s, "above"))
+                    settings->layer = 1;
+                else if (!strcmp(s, "below"))
+                    settings->layer = -1;
+                else
+                    settings->layer = 0;
+                g_free(s);
+            }
+
+            settings->iconic = -1;
+            if ((n = parse_find_node("iconic", app->children)))
+                settings->iconic = parse_bool(doc, n);
+
+            settings->skip_pager = -1;
+            if ((n = parse_find_node("skip_pager", app->children)))
+                settings->skip_pager = parse_bool(doc, n);
+
+            settings->skip_taskbar = -1;
+            if ((n = parse_find_node("skip_taskbar", app->children)))
+                settings->skip_taskbar = parse_bool(doc, n);
+
+            settings->fullscreen = -1;
+            if ((n = parse_find_node("fullscreen", app->children)))
+                settings->fullscreen = parse_bool(doc, n);
+
+            settings->max_horz = -1;
+            settings->max_vert = -1;
+            if ((n = parse_find_node("maximized", app->children))) {
+                gchar *s = parse_string(doc, n);
+                if (!strcmp(s, "horizontal")) {
+                    settings->max_horz = TRUE;
+                    settings->max_vert = FALSE;
+                } else if (!strcmp(s, "vertical")) {
+                    settings->max_horz = FALSE;
+                    settings->max_vert = TRUE;
+                } else
+                    settings->max_horz = settings->max_vert =
+                        parse_bool(doc, n);
+                g_free(s);
+            }
+
+            config_per_app_settings = g_slist_append(config_per_app_settings,
+                                              (gpointer) settings);
+        }
+        
+        app = parse_find_node("application", app->next);
+    }
+}
 
 /*
 
@@ -250,8 +432,12 @@ static void parse_theme(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
         g_free(config_title_layout);
         config_title_layout = parse_string(doc, n);
     }
+    if ((n = parse_find_node("titleNumber", node)))
+        config_title_number = parse_bool(doc, n);
     if ((n = parse_find_node("keepBorder", node)))
         config_theme_keepborder = parse_bool(doc, n);
+    if ((n = parse_find_node("hideDisabled", node)))
+        config_theme_hidedisabled = parse_bool(doc, n);
 }
 
 static void parse_desktops(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
@@ -298,6 +484,8 @@ static void parse_resize(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
     
     if ((n = parse_find_node("drawContents", node)))
         config_resize_redraw = parse_bool(doc, n);
+    if ((n = parse_find_node("fourCorner", node)))
+        config_resize_four_corners = parse_bool(doc, n);
     if ((n = parse_find_node("popupShow", node))) {
         config_resize_popup_show = parse_int(doc, n);
         if (parse_contains("Always", doc, n))
@@ -356,14 +544,17 @@ static void parse_dock(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
             config_dock_x = parse_int(doc, n);
         if ((n = parse_find_node("floatingY", node)))
             config_dock_y = parse_int(doc, n);
+    } else {
+        if ((n = parse_find_node("noStrut", node)))
+            config_dock_nostrut = parse_bool(doc, n);
     }
     if ((n = parse_find_node("stacking", node))) {
         if (parse_contains("top", doc, n))
-            config_dock_layer = OB_STACKING_LAYER_DOCK_ABOVE;
+            config_dock_layer = OB_STACKING_LAYER_ABOVE;
         else if (parse_contains("normal", doc, n))
-            config_dock_layer = OB_STACKING_LAYER_DOCK_NORMAL;
+            config_dock_layer = OB_STACKING_LAYER_NORMAL;
         else if (parse_contains("bottom", doc, n))
-            config_dock_layer = OB_STACKING_LAYER_DOCK_BELOW;
+            config_dock_layer = OB_STACKING_LAYER_BELOW;
     }
     if ((n = parse_find_node("direction", node))) {
         if (parse_contains("horizontal", doc, n))
@@ -375,6 +566,8 @@ static void parse_dock(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
         config_dock_hide = parse_bool(doc, n);
     if ((n = parse_find_node("hideDelay", node)))
         config_dock_hide_delay = parse_int(doc, n) * 1000;
+    if ((n = parse_find_node("showDelay", node)))
+        config_dock_show_delay = parse_int(doc, n) * 1000;
     if ((n = parse_find_node("moveButton", node))) {
         gchar *str = parse_string(doc, n);
         guint b, s;
@@ -405,10 +598,14 @@ static void parse_menu(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
             config_menu_warppointer = parse_bool(doc, n);
         if ((n = parse_find_node("xorStyle", node)))
             config_menu_xorstyle = parse_bool(doc, n);
-        if ((n = parse_find_node("hilightFirst", node)))
-            config_menu_hilightfirst = parse_bool(doc, n);
         if ((n = parse_find_node("hideDelay", node)))
             config_menu_hide_delay = parse_int(doc, n);
+        if ((n = parse_find_node("middle", node)))
+            config_menu_middle = parse_bool(doc, n);
+        if ((n = parse_find_node("submenuShowDelay", node)))
+            config_submenu_show_delay = parse_int(doc, n);
+        if ((n = parse_find_node("desktopMenuIcons", node)))
+            config_menu_client_list_icons = parse_bool(doc, n);
     }
 }
    
@@ -422,6 +619,8 @@ static void parse_resistance(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
         config_resist_win = parse_int(doc, n);
     if ((n = parse_find_node("screen_edge_strength", node)))
         config_resist_edge = parse_int(doc, n);
+    if ((n = parse_find_node("edges_hit_layers_below", node)))
+        config_resist_layers_below = parse_bool(doc, n);
 }
 
 typedef struct
@@ -544,7 +743,9 @@ void config_startup(ObParseInst *i)
     config_theme = NULL;
 
     config_title_layout = g_strdup("NLIMC");
+    config_title_number = TRUE;
     config_theme_keepborder = TRUE;
+    config_theme_hidedisabled = FALSE;
 
     parse_register(i, "theme", parse_theme, NULL);
 
@@ -555,19 +756,22 @@ void config_startup(ObParseInst *i)
     parse_register(i, "desktops", parse_desktops, NULL);
 
     config_resize_redraw = TRUE;
+    config_resize_four_corners = FALSE;
     config_resize_popup_show = 1; /* nonpixel increments */
     config_resize_popup_pos = 0;  /* center of client */
 
     parse_register(i, "resize", parse_resize, NULL);
 
-    config_dock_layer = OB_STACKING_LAYER_DOCK_ABOVE;
+    config_dock_layer = OB_STACKING_LAYER_ABOVE;
     config_dock_pos = OB_DIRECTION_NORTHEAST;
     config_dock_floating = FALSE;
+    config_dock_nostrut = FALSE;
     config_dock_x = 0;
     config_dock_y = 0;
     config_dock_orient = OB_ORIENTATION_VERT;
     config_dock_hide = FALSE;
     config_dock_hide_delay = 300;
+    config_dock_show_delay = 300;
     config_dock_app_move_button = 2; /* middle */
     config_dock_app_move_modifiers = 0;
 
@@ -589,16 +793,23 @@ void config_startup(ObParseInst *i)
 
     config_resist_win = 10;
     config_resist_edge = 20;
+    config_resist_layers_below = FALSE;
 
     parse_register(i, "resistance", parse_resistance, NULL);
 
     config_menu_warppointer = TRUE;
     config_menu_xorstyle = TRUE;
-    config_menu_hilightfirst = TRUE;
     config_menu_hide_delay = 250;
+    config_menu_middle = FALSE;
+    config_submenu_show_delay = 0;
+    config_menu_client_list_icons = TRUE;
     config_menu_files = NULL;
 
     parse_register(i, "menu", parse_menu, NULL);
+
+    config_per_app_settings = NULL;
+
+    parse_register(i, "applications", parse_per_app_settings, NULL);
 }
 
 void config_shutdown()
@@ -616,4 +827,13 @@ void config_shutdown()
     for (it = config_menu_files; it; it = g_slist_next(it))
         g_free(it->data);
     g_slist_free(config_menu_files);
+
+    for (it = config_per_app_settings; it; it = g_slist_next(it)) {
+        ObAppSettings *itd = (ObAppSettings *)it->data;
+        g_free(itd->name);
+        g_free(itd->role);
+        g_free(itd->class);
+        g_free(it->data);
+    }
+    g_slist_free(config_per_app_settings);
 }
