@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    config.h for the Openbox window manager
-   Copyright (c) 2004        Mikael Magnusson
+   Copyright (c) 2006        Mikael Magnusson
    Copyright (c) 2003        Ben Jansens
 
    This program is free software; you can redistribute it and/or modify
@@ -45,6 +45,9 @@ extern ObPlacePolicy config_place_policy;
 /*! When true windows' contents are refreshed while they are resized; otherwise
   they are not updated until the resize is complete */
 extern gboolean config_resize_redraw;
+/*! Divide windows in 4 or 9 areas when doing a resize. The middle will be move
+  when selecting 9 corners */
+extern gboolean config_resize_four_corners;
 /*! show move/resize popups? 0 = no, 1 = always, 2 = only
   resizing !1 increments */
 extern gint config_resize_popup_show;
@@ -55,6 +58,8 @@ extern gint config_resize_popup_pos;
 extern ObStackingLayer config_dock_layer;
 /*! Is the dock floating */
 extern gboolean config_dock_floating;
+/*! Don't use a strut for the dock */
+extern gboolean config_dock_nostrut;
 /*! Where to place the dock if not floating */
 extern ObDirection config_dock_pos;
 /*! If config_dock_floating, this is the top-left corner's
@@ -69,18 +74,24 @@ extern ObOrientation config_dock_orient;
 extern gboolean config_dock_hide;
 /*! The number of microseconds to wait before hiding the dock */
 extern guint config_dock_hide_delay;
+/*! The number of microseconds to wait before showing the dock */
+extern guint config_dock_show_delay;
 /*! The mouse button to be used to move dock apps */
 extern guint config_dock_app_move_button;
 /*! The modifiers to be used with the button to move dock apps */
 extern guint config_dock_app_move_modifiers;
 
-/* The name of the theme */
+/*! The name of the theme */
 extern gchar *config_theme;
 
-/* Show the onepixel border after toggleDecor */
+/*! Show the one-pixel border after toggleDecor */
 extern gboolean config_theme_keepborder;
-/* Titlebar button layout */
+/*! Hide window frame buttons that the window doesn't allow */
+extern gboolean config_theme_hidedisabled;
+/*! Titlebar button layout */
 extern gchar *config_title_layout;
+/*! Append a unique number to windows with same titles */
+extern gboolean config_title_number;
 
 /*! The number of desktops */
 extern gint config_desktops_num;
@@ -104,17 +115,25 @@ extern gint config_mouse_dclicktime;
 extern gint config_resist_win;
 /*! Number of pixels to resist while crossing a screen's edge */
 extern gint config_resist_edge;
+/*! Should windows resist edges at layers below */
+extern gboolean config_resist_layers_below;
 
 /*! Warp near edge on menu? */
 extern gboolean config_menu_warppointer;
 /*! make menus jump around a lot */
 extern gboolean config_menu_xorstyle;
-/*! hilight first entry in menu */
-extern gboolean config_menu_hilightfirst;
 /*! delay for hiding menu when opening */
 extern guint    config_menu_hide_delay;
+/*! Center menus vertically about the parent entry */
+extern gboolean config_menu_middle;
+/*! delay before opening a submenu */
+extern guint    config_submenu_show_delay;
+/*! show icons in client_list_menu */
+extern gboolean config_menu_client_list_icons;
 /*! User-specified menu files */
 extern GSList *config_menu_files;
+/*! Per app settings */
+extern GSList *config_per_app_settings;
 
 void config_startup(struct _ObParseInst *i);
 void config_shutdown();
