@@ -1,6 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    prop.c for the Openbox window manager
+   Copyright (c) 2006        Mikael Magnusson
    Copyright (c) 2003        Ben Jansens
 
    This program is free software; you can redistribute it and/or modify
@@ -206,7 +207,7 @@ static gboolean get_prealloc(Window win, Atom prop, Atom type, gint size,
                     data[i] = xdata[i];
                     break;
                 case 16:
-                    ((guint16*)data)[i] = ((guint16*)xdata)[i];
+                    ((guint16*)data)[i] = ((gushort*)xdata)[i];
                     break;
                 case 32:
                     ((guint32*)data)[i] = ((gulong*)xdata)[i];
@@ -245,7 +246,7 @@ static gboolean get_all(Window win, Atom prop, Atom type, gint size,
                     (*data)[i] = xdata[i];
                     break;
                 case 16:
-                    ((guint16*)*data)[i] = ((guint16*)xdata)[i];
+                    ((guint16*)*data)[i] = ((gushort*)xdata)[i];
                     break;
                 case 32:
                     ((guint32*)*data)[i] = ((gulong*)xdata)[i];
@@ -383,13 +384,13 @@ gboolean prop_get_strings_utf8(Window win, Atom prop, gchar ***ret)
     return FALSE;
 }
 
-void prop_set32(Window win, Atom prop, Atom type, guint32 val)
+void prop_set32(Window win, Atom prop, Atom type, gulong val)
 {
     XChangeProperty(ob_display, win, prop, type, 32, PropModeReplace,
                     (guchar*)&val, 1);
 }
 
-void prop_set_array32(Window win, Atom prop, Atom type, guint32 *val,
+void prop_set_array32(Window win, Atom prop, Atom type, gulong *val,
                       guint num)
 {
     XChangeProperty(ob_display, win, prop, type, 32, PropModeReplace,
