@@ -1,6 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    geom.h for the Openbox window manager
+   Copyright (c) 2006        Mikael Magnusson
    Copyright (c) 2003        Ben Jansens
 
    This program is free software; you can redistribute it and/or modify
@@ -16,8 +17,8 @@
    See the COPYING file for a copy of the GNU General Public License.
 */
 
-#ifndef __geom_h
-#define __geom_h
+#ifndef __render_geom_h
+#define __render_geom_h
 
 typedef struct {
     int width;
